(* ast.mli -- Abstract Syntax Tree for Parser *)

open Type

(* Expressions *)
type expr = 																		
    Literal of literal 						(* 3.14159 etc.  *)
  | TableAsTable of string * expr * expr * expr * expr (* represent table expression that looks like t[1~2,2~5],t[1~2,@] *)
  | TableAsLine_R of string * expr * expr * expr (* represent table expression that looks like t[4,3~5] *)
  | TableAsLine_L of string * expr * expr * expr (* represent table expression that looks like t[4,3~5] *)
  | TableAsCell of string * expr * expr (* represnet table expression that looks like t[5,3] *)
  | TableAsCellValue of string * expr * expr (* represnet table expression that looks like ^t[5,3] *)
  | LineAsLine of string * expr * expr (* represent line expression that looks like l[1~2] *)
  | LineAsCell of string * expr (* represent line expression that looks like l[4] *)
  | LineAsCellValue of string * expr (* represent line expression that looks like ^l[4] *)
  | CellAsCell of string  (* An ID for a cell.  Distinguised from other ID's in that it's prefixed by ^ *)
  | Id of string 									(* foo *)
  | Binop of expr * binop * expr 	(* a + b *)
  | Assign of expr * expr 				(* a = t[1,2~4] *)
  | Call of string * expr list 		(* foo(1, 25) *)
  | Uniop of uniop * expr 				(* not a *)
  | All 													(* string here can only be "@" - wildcard*)
  | Null 													(* for null assignment*)
  | UnaryMinus of expr 						(* -1 *)
  | NewTotalCell of string  (* An ID for a cell,table and line.  Distinguised from other ID's in that it's prefixed by ^ *)


(* Return type and Name of a function *)
type funcSig = FuncSig of taty_ret * string


type varSig = VarSig of taty_var * string


(*initialization of variables *)
(* taty*string --> data type * var name *)
type  init =                             
    NullInit of taty_var * string       							 				(*assign null/default value*)
  | VarInit of taty_var * string * expr 							 				(*assgin a specific value to the new variable*)
  | TableInit of taty_var * string * expr * expr * taty_var   (* e.g., table t = ([3,4], int) -- taty is only int or float*)


(* Statements  *)
type stmt = 
    Block of stmt list
  | Expr of expr   										(*foo = bar + 3; *)
  | Return of expr 										(* return 42 also includes return function_name *)
  | Print of expr 
  | If of expr * stmt * stmt 					(* if (foo == 42) {} else {} *)
  | For of expr * expr * expr * stmt 	(* for loop *)
  | While of expr * stmt 							(* while (i<10) { i = i + 1 } *)
  | NoElseMark												(* For loops without else*)
  | VarDecl of init
  | Break


(* Function Declarations *)
type func_decl = {
		fname : funcSig; 				(* Name and return type of the function *)
		formals : varSig list; 	(* Formal arguments, type & names *)
		body : stmt;
}


(* Bascially, the parts of a program *)
type construct = 
    GlobalVar of init
  | FuncDef of func_decl


type program = construct list (* global vars, funcs *)
